@@ -117,7 +117,7 @@
   (define-key m ">" #'go-guru-callees)
   (define-key m "x" #'go-guru-expand-region)) ;; x for expand
 
-(define-key go-mode-map (kbd "C-c C-o") #'go-guru-map)
+(define-key go-mode-map (kbd "C-c C-o") 'go-guru-map)
 
 (easy-menu-define go-guru-mode-menu go-mode-map
   "Menu for Go Guru."
@@ -138,6 +138,13 @@
     "---"
     ["Set pointer analysis scope..." go-guru-set-scope t]))
 
+(defun go-guru--read-scope ()
+  "Read go-guru-scope from the minibuffer."
+  (completing-read-multiple "guru-scope (comma-separated): "
+                            (go-packages) nil nil nil 'go-guru--scope-history))
+
+(eval-when-compile (require 'subr-x))
+
 ;;;###autoload
 (defun go-guru-set-scope ()
   "Set the scope for the Go guru, prompting the user to edit the previous scope.
@@ -152,14 +159,8 @@ A pattern preceded by '-' is negative, so the scope
 	encoding/...,-encoding/xml
 matches all encoding packages except encoding/xml."
   (interactive)
-  (let ((scope (read-from-minibuffer "Go guru scope: "
-				     go-guru-scope
-				     nil
-				     nil
-				     'go-guru--scope-history)))
-    (if (string-equal "" scope)
-	(error "You must specify a non-empty scope for the Go guru"))
-    (setq go-guru-scope scope)))
+  (let ((scope (go-guru--read-scope)))
+    (setq go-guru-scope (string-join scope ","))))
 
 (defun go-guru--set-scope-if-empty ()
   (if (string-equal "" go-guru-scope)
@@ -280,7 +281,7 @@ effective name of the current buffer."
 	 (cmd (append (list go-guru-command
 			    "-modified"
 			    "-scope" go-guru-scope
-			    (format "-tags=%s" (mapconcat 'identity go-guru-build-tags ",")))
+			    (format "-tags=%s" (mapconcat 'identity go-guru-build-tags " ")))
 		      flags
 		      (list mode
 			    posn))))
@@ -319,11 +320,11 @@ If BUFFER, return the number of characters in that buffer instead."
   "Go to the OFFSETth byte in the current line."
   (goto-char (byte-to-position (+ (position-bytes (point-at-bol)) (1- offset)))))
 
-(defun go-guru--goto-pos (posn)
+(defun go-guru--goto-pos (posn other-window)
   "Find the file containing the position POSN (of the form `file:line:col')
 set the point to it, switching the current buffer."
   (let ((file-line-pos (split-string posn ":")))
-    (find-file (car file-line-pos))
+    (funcall (if other-window #'find-file-other-window #'find-file) (car file-line-pos))
     (goto-char (point-min))
     (forward-line (1- (string-to-number (cadr file-line-pos))))
     (go-guru--goto-byte-column (string-to-number (cl-caddr file-line-pos)))))
@@ -359,7 +360,7 @@ function containing the current point."
   (go-guru--start "callstack"))
 
 ;;;###autoload
-(defun go-guru-definition ()
+(defun go-guru-definition (&optional other-window)
   "Jump to the definition of the selected identifier."
   (interactive)
   (or buffer-file-name
@@ -367,10 +368,19 @@ function containing the current point."
   (let* ((res (go-guru--json "definition"))
 	 (desc (cdr (assoc 'desc res))))
     (push-mark)
-    (ring-insert find-tag-marker-ring (point-marker))
-    (go-guru--goto-pos (cdr (assoc 'objpos res)))
+    (if (eval-when-compile (fboundp 'xref-push-marker-stack))
+        ;; TODO: Integrate this facility with XRef.
+        (xref-push-marker-stack)
+      (ring-insert find-tag-marker-ring (point-marker)))
+    (go-guru--goto-pos (cdr (assoc 'objpos res)) other-window)
     (message "%s" desc)))
 
+;;;###autoload
+(defun go-guru-definition-other-window ()
+  "Jump to the defintion of the selected identifier in another window"
+  (interactive)
+  (go-guru-definition t))
+
 ;;;###autoload
 (defun go-guru-describe ()
   "Describe the selected syntax, its kind, type and methods."
@@ -450,6 +460,22 @@ highlights from previously highlighted identifier."
   (go-guru-unhighlight-identifiers)
   (go-guru--hl-identifier))
 
+;;;###autoload
+(define-minor-mode go-guru-hl-identifier-mode
+  "Highlight instances of the identifier at point after a short
+timeout."
+  :group 'go-guru
+  (if go-guru-hl-identifier-mode
+      (progn
+	(go-guru--hl-set-timer)
+	;; Unhighlight if point moves off identifier
+	(add-hook 'post-command-hook #'go-guru--hl-identifiers-post-command-hook nil t)
+	;; Unhighlight any time the buffer changes
+	(add-hook 'before-change-functions #'go-guru--hl-identifiers-before-change-function nil t))
+    (remove-hook 'post-command-hook #'go-guru--hl-identifiers-post-command-hook t)
+    (remove-hook 'before-change-functions #'go-guru--hl-identifiers-before-change-function t)
+    (go-guru-unhighlight-identifiers)))
+
 (defun go-guru--hl-identifier ()
   "Highlight all instances of the identifier under point."
   (let ((posn (cdr (assoc 'sameids (go-guru-what)))))
@@ -478,22 +504,6 @@ identifier at point, if necessary."
 				      t
 				      #'go-guru--hl-identifiers-function)))
 
-;;;###autoload
-(define-minor-mode go-guru-hl-identifier-mode
-  "Highlight instances of the identifier at point after a short
-timeout."
-  :group 'go-guru
-  (if go-guru-hl-identifier-mode
-      (progn
-	(go-guru--hl-set-timer)
-	;; Unhighlight if point moves off identifier
-	(add-hook 'post-command-hook #'go-guru--hl-identifiers-post-command-hook nil t)
-	;; Unhighlight any time the buffer changes
-	(add-hook 'before-change-functions #'go-guru--hl-identifiers-before-change-function nil t))
-    (remove-hook 'post-command-hook #'go-guru--hl-identifiers-post-command-hook t)
-    (remove-hook 'before-change-functions #'go-guru--hl-identifiers-before-change-function t)
-    (go-guru-unhighlight-identifiers)))
-
 (defun go-guru--on-overlay-p (id)
   "Return whether point is on a guru overlay of type ID."
   (cl-find-if (lambda (el) (eq (overlay-get el 'go-guru-overlay) id)) (overlays-at (point))))
@@ -543,9 +553,9 @@ end point."
 
 (provide 'go-guru)
 
-;; Local variables:
+;; Local Variables:
 ;; indent-tabs-mode: t
 ;; tab-width: 8
-;; End
+;; End:
 
 ;;; go-guru.el ends here
